@@ -35,7 +35,9 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <dirent.h>
+#ifdef MHD_HAVE_LIBMAGIC
 #include <magic.h>
+#endif /* MHD_HAVE_LIBMAGIC */
 #include <limits.h>
 #include <ctype.h>
 
@@ -52,12 +54,14 @@
  */
 #define NUMBER_OF_THREADS CPU_COUNT
 
+#ifdef MHD_HAVE_LIBMAGIC
 /**
  * How many bytes of a file do we give to libmagic to determine the mime type?
  * 16k might be a bit excessive, but ought not hurt performance much anyway,
  * and should definitively be on the safe side.
  */
 #define MAGIC_HEADER_SIZE (16 * 1024)
+#endif /* MHD_HAVE_LIBMAGIC */
 
 
 /**
@@ -183,10 +187,12 @@ static struct MHD_Response *request_refused_response;
  */
 static pthread_mutex_t mutex;
 
+#ifdef MHD_HAVE_LIBMAGIC
 /**
  * Global handle to MAGIC data.
  */
 static magic_t magic;
+#endif /* MHD_HAVE_LIBMAGIC */
 
 
 /**
@@ -248,7 +254,7 @@ struct ResponseDataContext
  *
  * @param rdc where to store the list of files
  * @param dirname name of the directory to list
- * @return MHD_YES on success, MHD_NO on error
+ * @return #MHD_YES on success, #MHD_NO on error
  */
 static int
 list_directory (struct ResponseDataContext *rdc,
@@ -265,7 +271,7 @@ list_directory (struct ResponseDataContext *rdc,
     {
       if ('.' == de->d_name[0])
 	continue;
-      if (sizeof (fullname) <= (size_t)
+      if (sizeof (fullname) <= (unsigned int)
 	  snprintf (fullname, sizeof (fullname),
 		    "%s/%s",
 		    dirname, de->d_name))
@@ -427,7 +433,7 @@ struct UploadContext
  * @param ret string to update, NULL or 0-terminated
  * @param data data to append
  * @param size number of bytes in 'data'
- * @return MHD_NO on allocation failure, MHD_YES on success
+ * @return #MHD_NO on allocation failure, #MHD_YES on success
  */
 static int
 do_append (char **ret,
@@ -441,13 +447,18 @@ do_append (char **ret,
     old_len = 0;
   else
     old_len = strlen (*ret);
-  buf = malloc (old_len + size + 1);
-  if (NULL == buf)
+  if (NULL == (buf = malloc (old_len + size + 1)))
     return MHD_NO;
-  memcpy (buf, *ret, old_len);
   if (NULL != *ret)
-    free (*ret);
-  memcpy (&buf[old_len], data, size);
+    {
+      memcpy (buf,
+	      *ret,
+	      old_len);
+      free (*ret);
+    }
+  memcpy (&buf[old_len],
+	  data,
+	  size);
   buf[old_len + size] = '\0';
   *ret = buf;
   return MHD_YES;
@@ -470,8 +481,8 @@ do_append (char **ret,
  *              specified offset
  * @param off offset of data in the overall value
  * @param size number of bytes in data available
- * @return MHD_YES to continue iterating,
- *         MHD_NO to abort the iteration
+ * @return #MHD_YES to continue iterating,
+ *         #MHD_NO to abort the iteration
  */
 static int
 process_upload_data (void *cls,
@@ -486,6 +497,10 @@ process_upload_data (void *cls,
 {
   struct UploadContext *uc = cls;
   int i;
+  (void)kind;              /* Unused. Silent compiler warning. */
+  (void)content_type;      /* Unused. Silent compiler warning. */
+  (void)transfer_encoding; /* Unused. Silent compiler warning. */
+  (void)off;               /* Unused. Silent compiler warning. */
 
   if (0 == strcmp (key, "category"))
     return do_append (&uc->category, data, size);
@@ -545,7 +560,7 @@ process_upload_data (void *cls,
 		uc->category,
 		filename);
       for (i=strlen (fn)-1;i>=0;i--)
-	if (! isprint ((int) fn[i]))
+	if (! isprint ((unsigned char) fn[i]))
 	  fn[i] = '_';
       uc->fd = open (fn,
 		     O_CREAT | O_EXCL
@@ -606,6 +621,9 @@ response_completed_callback (void *cls,
 			     enum MHD_RequestTerminationCode toe)
 {
   struct UploadContext *uc = *con_cls;
+  (void)cls;         /* Unused. Silent compiler warning. */
+  (void)connection;  /* Unused. Silent compiler warning. */
+  (void)toe;         /* Unused. Silent compiler warning. */
 
   if (NULL == uc)
     return; /* this request wasn't an upload request */
@@ -665,9 +683,9 @@ return_directory_response (struct MHD_Connection *connection)
  * @param method GET, PUT, POST, etc.
  * @param version HTTP version
  * @param upload_data data from upload (PUT/POST)
- * @param upload_data_size number of bytes in "upload_data"
+ * @param upload_data_size number of bytes in @a upload_data
  * @param ptr our context
- * @return MHD_YES on success, MHD_NO to drop connection
+ * @return #MHD_YES on success, #MHD_NO to drop connection
  */
 static int
 generate_page (void *cls,
@@ -682,12 +700,16 @@ generate_page (void *cls,
   int ret;
   int fd;
   struct stat buf;
+  (void) cls;               /* Unused. Silent compiler warning. */
+  (void) version;           /* Unused. Silent compiler warning. */
 
   if (0 != strcmp (url, "/"))
     {
       /* should be file download */
+#ifdef MHD_HAVE_LIBMAGIC
       char file_data[MAGIC_HEADER_SIZE];
       ssize_t got;
+#endif /* MHD_HAVE_LIBMAGIC */
       const char *mime;
 
       if ( (0 != strcmp (method, MHD_HTTP_METHOD_GET)) &&
@@ -710,13 +732,15 @@ generate_page (void *cls,
 	return MHD_queue_response (connection,
 				   MHD_HTTP_NOT_FOUND,
 				   file_not_found_response);
+#ifdef MHD_HAVE_LIBMAGIC
       /* read beginning of the file to determine mime type  */
       got = read (fd, file_data, sizeof (file_data));
+      (void) lseek (fd, 0, SEEK_SET);
       if (-1 != got)
 	mime = magic_buffer (magic, file_data, got);
       else
+#endif /* MHD_HAVE_LIBMAGIC */
 	mime = NULL;
-      (void) lseek (fd, 0, SEEK_SET);
 
       if (NULL == (response = MHD_create_response_from_fd (buf.st_size,
 							   fd)))
@@ -804,6 +828,7 @@ generate_page (void *cls,
 }
 
 
+#ifndef MINGW
 /**
  * Function called if we get a SIGPIPE. Does nothing.
  *
@@ -812,6 +837,7 @@ generate_page (void *cls,
 static void
 catcher (int sig)
 {
+  (void)sig;	/* Unused. Silent compiler warning. */
   /* do nothing */
 }
 
@@ -819,7 +845,6 @@ catcher (int sig)
 /**
  * setup handlers to ignore SIGPIPE.
  */
-#ifndef MINGW
 static void
 ignore_sigpipe ()
 {
@@ -866,8 +891,10 @@ main (int argc, char *const *argv)
 #ifndef MINGW
   ignore_sigpipe ();
 #endif
+#ifdef MHD_HAVE_LIBMAGIC
   magic = magic_open (MAGIC_MIME_TYPE);
   (void) magic_load (magic, NULL);
+#endif /* MHD_HAVE_LIBMAGIC */
 
   (void) pthread_mutex_init (&mutex, NULL);
   file_not_found_response = MHD_create_response_from_buffer (strlen (FILE_NOT_FOUND_PAGE),
@@ -905,7 +932,9 @@ main (int argc, char *const *argv)
   MHD_destroy_response (internal_error_response);
   update_cached_response (NULL);
   (void) pthread_mutex_destroy (&mutex);
+#ifdef MHD_HAVE_LIBMAGIC
   magic_close (magic);
+#endif /* MHD_HAVE_LIBMAGIC */
   return 0;
 }
 
