@@ -28,7 +28,9 @@
 #include "microhttpd.h"
 #include <sys/stat.h>
 #include <limits.h>
+#ifdef MHD_HTTPS_REQUIRE_GRYPT
 #include <gcrypt.h>
+#endif /* MHD_HTTPS_REQUIRE_GRYPT */
 #include "tls_test_common.h"
 
 extern const char srv_key_pem[];
@@ -41,10 +43,11 @@ int curl_check_version (const char *req_version, ...);
  *
  */
 static int
-test_unmatching_ssl_version (void * cls, const char *cipher_suite,
+test_unmatching_ssl_version (void * cls, int port, const char *cipher_suite,
                              int curl_req_ssl_version)
 {
   struct CBC cbc;
+  (void)cls;    /* Unused. Silent compiler warning. */
   if (NULL == (cbc.buf = malloc (sizeof (char) * 256)))
     {
       fprintf (stderr, "Error: failed to allocate: %s\n",
@@ -55,10 +58,13 @@ test_unmatching_ssl_version (void * cls, const char *cipher_suite,
   cbc.pos = 0;
 
   char url[255];
-  if (gen_test_file_url (url, DEAMON_TEST_PORT))
+  if (gen_test_file_url (url,
+                         sizeof (url),
+                         port))
     {
       free (cbc.buf);
-      fprintf (stderr, "Internal error in gen_test_file_url\n");
+      fprintf (stderr,
+               "Internal error in gen_test_file_url\n");
       return -1;
     }
 
@@ -84,12 +90,21 @@ main (int argc, char *const *argv)
   const char *ssl_version;
   int daemon_flags =
     MHD_USE_THREAD_PER_CONNECTION | MHD_USE_INTERNAL_POLLING_THREAD | MHD_USE_TLS | MHD_USE_ERROR_LOG;
+  int port;
+  (void)argc; (void)argv;       /* Unused. Silent compiler warning. */
 
+  if (MHD_NO != MHD_is_feature_supported (MHD_FEATURE_AUTODETECT_BIND_PORT))
+    port = 0;
+  else
+    port = 3010;
+
+#ifdef MHD_HTTPS_REQUIRE_GRYPT
   gcry_control (GCRYCTL_DISABLE_SECMEM, 0);
   gcry_control (GCRYCTL_ENABLE_QUICK_RANDOM, 0);
 #ifdef GCRYCTL_INITIALIZATION_FINISHED
   gcry_control (GCRYCTL_INITIALIZATION_FINISHED, 0);
 #endif
+#endif /* MHD_HTTPS_REQUIRE_GRYPT */
  if (curl_check_version (MHD_REQ_CURL_VERSION))
     {
       return 77;
@@ -106,11 +121,8 @@ main (int argc, char *const *argv)
     return 77;
   }
 
-  if (0 != curl_global_init (CURL_GLOBAL_ALL))
-    {
-      fprintf (stderr, "Error: %s\n", strerror (errno));
-      return 99;
-    }
+  if (!testsuite_curl_global_init ())
+    return 99;
 
   const char *aes128_sha = "AES128-SHA";
   const char *aes256_sha = "AES256-SHA";
@@ -123,7 +135,7 @@ main (int argc, char *const *argv)
 
   if (0 !=
     test_wrap ("TLS1.0-AES-SHA1",
-	       &test_https_transfer, NULL, daemon_flags,
+	       &test_https_transfer, NULL, port, daemon_flags,
 	       aes128_sha,
 	       CURL_SSLVERSION_TLSv1,
 	       MHD_OPTION_HTTPS_MEM_KEY, srv_key_pem,
@@ -138,7 +150,7 @@ main (int argc, char *const *argv)
 	   "The following handshake should fail (and print an error message)...\n");
   if (0 !=
     test_wrap ("TLS1.0 vs SSL3",
-	       &test_unmatching_ssl_version, NULL, daemon_flags,
+	       &test_unmatching_ssl_version, NULL, port, daemon_flags,
 	       aes256_sha,
 	       CURL_SSLVERSION_SSLv3,
 	       MHD_OPTION_HTTPS_MEM_KEY, srv_key_pem,
